@@ -1,19 +1,20 @@
+use datafusion::logical_expr::expr::Sort;
 use datafusion::prelude::{Column, Expr};
 
 pub fn column_sort_expr_desc(column: String) -> Expr {
-    Expr::Sort {
+    Expr::Sort(Sort {
         expr: Box::new(Expr::Column(Column::from_name(column))),
         asc: false,
         nulls_first: true,
-    }
+    })
 }
 
 pub fn column_sort_expr_asc(column: impl Into<String>) -> Expr {
-    Expr::Sort {
+    Expr::Sort(Sort {
         expr: Box::new(Expr::Column(Column::from_name(column))),
         asc: true,
         nulls_first: true,
-    }
+    })
 }
 
 pub mod graphql;
