@@ -1,4 +1,3 @@
-use std::convert::TryFrom;
 use std::ffi::OsStr;
 use std::io::Read;
 use std::path::Path;
@@ -7,6 +6,7 @@ use std::sync::Arc;
 use datafusion::datasource::TableProvider;
 
 use datafusion::arrow;
+use datafusion::datasource::listing::{ListingOptions, ListingTableUrl};
 use serde::de::{Deserialize, Deserializer};
 use serde_derive::Deserialize;
 use snafu::prelude::*;
@@ -20,11 +20,11 @@ pub mod arrow_ipc_stream;
 pub mod csv;
 pub mod database;
 pub mod delta;
+pub mod excel;
 pub mod google_spreadsheets;
 pub mod json;
 pub mod ndjson;
 pub mod parquet;
-pub mod xlsx;
 
 #[derive(Debug, Snafu)]
 pub enum Error {
@@ -44,8 +44,8 @@ pub enum Error {
     LoadArrowIpcFile { source: arrow_ipc_file::Error },
     #[snafu(display("Failed to load Google Sheet data: {source}"))]
     LoadGoogleSheet { source: google_spreadsheets::Error },
-    #[snafu(display("Failed to load XLSX data: {source}"))]
-    LoadXlsx { source: xlsx::Error },
+    #[snafu(display("Failed to load Excel data: {source}"))]
+    LoadExcel { source: excel::Error },
     #[snafu(display("Failed to load database data: {source}"))]
     LoadDatabase { source: database::Error },
     #[snafu(display("Failed to cast IO source to memory bytes for source: {table_source}"))]
@@ -85,6 +85,10 @@ pub enum Error {
     MissingOption {},
     #[snafu(display("Generic error: {msg}"))]
     Generic { msg: String },
+    #[snafu(display("Failed to infer table schema: {source}"))]
+    InferSchema {
+        source: datafusion::error::DataFusionError,
+    },
 }
 
 #[derive(Deserialize, Clone, Debug, Eq, PartialEq)]
@@ -148,14 +152,14 @@ pub struct TableOptionGoogleSpreadsheet {
 #[derive(Deserialize, Debug, Clone, Eq, PartialEq)]
 pub struct TableOptionCsv {
     #[serde(default = "TableOptionCsv::default_has_header")]
-    has_header: bool,
+    pub has_header: bool,
     #[serde(default = "TableOptionCsv::default_delimiter")]
     #[serde(deserialize_with = "TableOptionCsv::deserialize_delimiter")]
-    delimiter: u8,
+    pub delimiter: u8,
     #[serde(default = "TableOptionCsv::default_projection")]
-    projection: Option<Vec<usize>>,
+    pub projection: Option<Vec<usize>>,
     #[serde(default = "TableOptionCsv::default_use_memory_table")]
-    use_memory_table: bool,
+    pub use_memory_table: bool,
 }
 
 impl TableOptionCsv {
@@ -188,6 +192,13 @@ impl TableOptionCsv {
         self
     }
 
+    #[inline]
+    #[must_use]
+    pub fn with_use_memory_table(mut self, use_memory: bool) -> Self {
+        self.use_memory_table = use_memory;
+        self
+    }
+
     fn deserialize_delimiter<'de, D>(deserializer: D) -> Result<u8, D::Error>
     where
         D: Deserializer<'de>,
@@ -200,6 +211,7 @@ impl TableOptionCsv {
             )),
         }
     }
+
     #[inline]
     pub fn default_use_memory_table() -> bool {
         true
@@ -220,7 +232,7 @@ impl Default for TableOptionCsv {
 #[derive(Deserialize, Debug, Clone, Eq, PartialEq)]
 pub struct TableOptionParquet {
     #[serde(default = "TableOptionParquet::default_use_memory_table")]
-    use_memory_table: bool,
+    pub use_memory_table: bool,
 }
 
 impl TableOptionParquet {
@@ -238,15 +250,20 @@ impl Default for TableOptionParquet {
     }
 }
 
-#[derive(Deserialize, Debug, Clone, Eq, PartialEq)]
-pub struct TableOptionXlsx {
-    sheet_name: Option<String>,
+#[derive(Deserialize, Default, Debug, Clone, Eq, PartialEq)]
+pub struct TableOptionExcel {
+    pub sheet_name: Option<String>,
+    pub rows_range_start: Option<usize>,
+    pub rows_range_end: Option<usize>,
+    pub columns_range_start: Option<usize>,
+    pub columns_range_end: Option<usize>,
+    pub schema_inference_lines: Option<usize>,
 }
 
 #[derive(Deserialize, Debug, Clone, Eq, PartialEq)]
 pub struct TableOptionDelta {
     #[serde(default = "TableOptionDelta::default_use_memory_table")]
-    use_memory_table: bool,
+    pub use_memory_table: bool,
 }
 
 impl TableOptionDelta {
@@ -285,7 +302,10 @@ pub enum TableLoadOption {
     jsonl {},
     parquet(TableOptionParquet),
     google_spreadsheet(TableOptionGoogleSpreadsheet),
-    xlsx(TableOptionXlsx),
+    xls(TableOptionExcel),
+    xlsx(TableOptionExcel),
+    xlsb(TableOptionExcel),
+    ods(TableOptionExcel),
     delta(TableOptionDelta),
     arrow {},
     arrows {},
@@ -295,7 +315,7 @@ pub enum TableLoadOption {
 }
 
 impl TableLoadOption {
-    fn as_google_spreadsheet(&self) -> Result<&TableOptionGoogleSpreadsheet, Error> {
+    pub fn as_google_spreadsheet(&self) -> Result<&TableOptionGoogleSpreadsheet, Error> {
         match self {
             Self::google_spreadsheet(opt) => Ok(opt),
             _ => Err(Error::ExpectFormatOption {
@@ -304,28 +324,28 @@ impl TableLoadOption {
         }
     }
 
-    fn as_xlsx(&self) -> Result<&TableOptionXlsx, Error> {
+    pub fn as_excel(&self) -> Result<&TableOptionExcel, Error> {
         match self {
-            Self::xlsx(opt) => Ok(opt),
-            _ => Err(Error::ExpectFormatOption { fmt: "xlsx" }),
+            Self::xls(opt) | Self::xlsx(opt) | Self::xlsb(opt) | Self::ods(opt) => Ok(opt),
+            _ => Err(Error::ExpectFormatOption { fmt: "excel" }),
         }
     }
 
-    fn as_csv(&self) -> Result<&TableOptionCsv, Error> {
+    pub fn as_csv(&self) -> Result<&TableOptionCsv, Error> {
         match self {
             Self::csv(opt) => Ok(opt),
             _ => Err(Error::ExpectFormatOption { fmt: "csv" }),
         }
     }
 
-    fn as_parquet(&self) -> Result<&TableOptionParquet, Error> {
+    pub fn as_parquet(&self) -> Result<&TableOptionParquet, Error> {
         match self {
             Self::parquet(opt) => Ok(opt),
             _ => Err(Error::ExpectFormatOption { fmt: "parquet" }),
         }
     }
 
-    fn as_delta(&self) -> Result<&TableOptionDelta, Error> {
+    pub fn as_delta(&self) -> Result<&TableOptionDelta, Error> {
         match self {
             Self::delta(opt) => Ok(opt),
             _ => Err(Error::ExpectFormatOption { fmt: "delta" }),
@@ -340,7 +360,10 @@ impl TableLoadOption {
             Self::csv { .. } => "csv",
             Self::parquet { .. } => "parquet",
             Self::google_spreadsheet(_) | Self::delta { .. } => "",
+            Self::xls { .. } => "xls",
             Self::xlsx { .. } => "xlsx",
+            Self::ods { .. } => "ods",
+            Self::xlsb { .. } => "xlsb",
             Self::arrow { .. } => "arrow",
             Self::arrows { .. } => "arrows",
             Self::mysql { .. } => "mysql",
@@ -390,9 +413,11 @@ pub struct TableSource {
     #[serde(flatten)]
     pub io_source: TableIoSource,
     pub schema: Option<TableSchema>,
+    pub schema_from_files: Option<Vec<String>>,
     pub option: Option<TableLoadOption>,
     #[serde(default = "TableSource::default_batch_size")]
     pub batch_size: usize,
+    pub partition_columns: Option<Vec<TableColumn>>,
 }
 
 impl From<KeyValueSource> for TableSource {
@@ -401,8 +426,10 @@ impl From<KeyValueSource> for TableSource {
             name: kv.name,
             io_source: kv.io_source,
             schema: kv.schema,
+            schema_from_files: kv.schema_from_files,
             option: kv.option,
             batch_size: Self::default_batch_size(),
+            partition_columns: None,
         }
     }
 }
@@ -415,11 +442,21 @@ impl TableSource {
             name: name.into(),
             io_source,
             schema: None,
+            schema_from_files: None,
             option,
             batch_size: Self::default_batch_size(),
+            partition_columns: None,
         }
     }
 
+    pub fn datafusion_partition_cols(&self) -> Option<Vec<(String, arrow::datatypes::DataType)>> {
+        self.partition_columns.as_ref().map(|cols| {
+            cols.iter()
+                .map(|col| (col.name.to_string(), col.data_type.clone()))
+                .collect::<Vec<_>>()
+        })
+    }
+
     pub fn new_with_uri(name: impl Into<String>, uri: impl Into<String>) -> Self {
         Self::new(name, uri.into())
     }
@@ -443,6 +480,18 @@ impl TableSource {
         self
     }
 
+    #[inline]
+    #[must_use]
+    pub fn with_partition_columns(mut self, partitions: Vec<TableColumn>) -> Self {
+        self.partition_columns = Some(partitions);
+        self
+    }
+
+    pub fn with_schema_from_files(mut self, files: Vec<String>) -> Self {
+        self.schema_from_files = Some(files);
+        self
+    }
+
     pub fn get_uri_str(&self) -> &str {
         match &self.io_source {
             TableIoSource::Uri(uri) => uri.as_str(),
@@ -498,7 +547,7 @@ impl TableSource {
                 match Path::new(uri).extension().and_then(OsStr::to_str) {
                     Some(ext) => match ext {
                         "csv" | "json" | "ndjson" | "jsonl" | "parquet" | "arrow" | "arrows"
-                        | "xlsx" => ext,
+                        | "xls" | "xlsx" | "xlsb" | "ods" => ext,
                         "sqlite" | "sqlite3" | "db" => "sqlite",
                         _ => {
                             return Err(Error::Extension {
@@ -530,6 +579,50 @@ impl TableSource {
     }
 }
 
+pub async fn datafusion_get_or_infer_schema(
+    dfctx: &datafusion::execution::context::SessionContext,
+    table_url: &ListingTableUrl,
+    listing_options: &ListingOptions,
+    schema: &Option<TableSchema>,
+    schema_from_files: &Option<Vec<String>>,
+) -> Result<arrow::datatypes::SchemaRef, Error> {
+    Ok(match (schema, schema_from_files) {
+        (Some(s), _) => Arc::new(s.into()),
+        (None, Some(files)) => {
+            if files.is_empty() {
+                return Err(Error::Generic {
+                    msg: "schema_from_files is an empty list".to_string(),
+                });
+            }
+            let mut schemas = vec![];
+            let table_root = Path::new(table_url.as_str());
+            for f in files.iter() {
+                let file_url = ListingTableUrl::parse(
+                    table_root
+                        .join(f.as_str())
+                        .to_str()
+                        .expect("Failed to create file url"),
+                )
+                .context(InferSchemaSnafu)?;
+                schemas.push(
+                    Arc::into_inner(
+                        listing_options
+                            .infer_schema(&dfctx.state(), &file_url)
+                            .await
+                            .context(InferSchemaSnafu)?,
+                    )
+                    .expect("Failed to unwrap schemaref into schema on merge"),
+                );
+            }
+            Arc::new(arrow::datatypes::Schema::try_merge(schemas).context(MergeSchemaSnafu)?)
+        }
+        (None, None) => listing_options
+            .infer_schema(&dfctx.state(), table_url)
+            .await
+            .context(InferSchemaSnafu)?,
+    })
+}
+
 pub async fn load(
     t: &TableSource,
     dfctx: &datafusion::execution::context::SessionContext,
@@ -545,7 +638,10 @@ pub async fn load(
             TableLoadOption::google_spreadsheet(_) => {
                 Arc::new(google_spreadsheets::to_mem_table(t).await?)
             }
-            TableLoadOption::xlsx { .. } => Arc::new(xlsx::to_mem_table(t).await?),
+            TableLoadOption::xlsx { .. }
+            | TableLoadOption::xls { .. }
+            | TableLoadOption::xlsb { .. }
+            | TableLoadOption::ods { .. } => Arc::new(excel::to_mem_table(t).await?),
             TableLoadOption::delta { .. } => delta::to_datafusion_table(t, dfctx).await?,
             TableLoadOption::arrow { .. } => {
                 Arc::new(arrow_ipc_file::to_mem_table(t, dfctx).await?)
@@ -569,6 +665,7 @@ pub async fn load(
             "json" => Arc::new(json::to_mem_table(t, dfctx).await?),
             "ndjson" | "jsonl" => Arc::new(ndjson::to_mem_table(t, dfctx).await?),
             "parquet" => parquet::to_datafusion_table(t, dfctx).await?,
+            "xls" | "xlsx" | "xlsb" | "ods" => Arc::new(excel::to_mem_table(t).await?),
             "arrow" => Arc::new(arrow_ipc_file::to_mem_table(t, dfctx).await?),
             "arrows" => Arc::new(arrow_ipc_stream::to_mem_table(t, dfctx).await?),
             "mysql" => Arc::new(database::DatabaseLoader::MySQL.to_mem_table(t)?),
@@ -662,6 +759,7 @@ pub struct KeyValueSource {
     #[serde(flatten)]
     pub io_source: TableIoSource,
     pub schema: Option<TableSchema>,
+    pub schema_from_files: Option<Vec<String>>,
     pub option: Option<TableLoadOption>,
 }
 
@@ -678,6 +776,7 @@ impl KeyValueSource {
             value: value.into(),
             io_source: source.into(),
             schema: None,
+            schema_from_files: None,
             option: None,
         }
     }
@@ -740,6 +839,28 @@ batch_size: 512
         assert_eq!(table_source.batch_size, 512);
     }
 
+    #[test]
+    fn deserialize_datetime() {
+        let table_source: TableSource = serde_yaml::from_str(
+            r#"
+name: "ts_table"
+uri: "test_data/a.parquet"
+schema:
+  columns:
+    - name: "ts"
+      data_type: !Timestamp [!Second, null]
+"#,
+        )
+        .unwrap();
+
+        use arrow::datatypes::*;
+
+        assert_eq!(
+            DataType::Timestamp(TimeUnit::Second, None),
+            table_source.schema.unwrap().columns[0].data_type,
+        );
+    }
+
     #[test]
     fn test_parse_table_uri() {
         let t = parse_table_uri_arg("t=a/b/c").unwrap();
@@ -767,6 +888,8 @@ batch_size: 512
     #[cfg(feature = "database-sqlite")]
     #[tokio::test]
     async fn test_load_sqlite_table() {
+        use datafusion::common::stats::Precision;
+
         let t = TableSource::new("uk_cities", "sqlite://../test_data/sqlite/sample.db");
         let ctx = datafusion::prelude::SessionContext::new();
         let table = load(&t, &ctx).await.unwrap();
@@ -774,13 +897,16 @@ batch_size: 512
             .scan(&ctx.state(), None, &[], None)
             .await
             .unwrap()
-            .statistics();
-        assert_eq!(stats.num_rows, Some(37));
+            .statistics()
+            .unwrap();
+        assert_eq!(stats.num_rows, Precision::Exact(37));
     }
 
     #[cfg(feature = "database-sqlite")]
     #[tokio::test]
     async fn test_load_sqlite_table_with_config() {
+        use datafusion::common::stats::Precision;
+
         for ext in vec!["db", "sqlite", "sqlite3"] {
             let t: TableSource = serde_yaml::from_str(&format!(
                 r#"
@@ -796,8 +922,9 @@ uri: "sqlite://../test_data/sqlite/sample.{}"
                 .scan(&ctx.state(), None, &[], None)
                 .await
                 .unwrap()
-                .statistics();
-            assert_eq!(stats.num_rows, Some(37));
+                .statistics()
+                .unwrap();
+            assert_eq!(stats.num_rows, Precision::Exact(37));
         }
     }
 }
