@@ -1,16 +1,18 @@
 use std::collections::HashMap;
-use std::sync::Arc;
 
 use async_trait::async_trait;
+use columnq::arrow::record_batch::RecordBatch;
 use columnq::datafusion::arrow;
 use columnq::datafusion::dataframe::DataFrame;
 use columnq::datafusion::error::DataFusionError;
+use columnq::datafusion::prelude::SessionContext;
 use columnq::encoding;
 use columnq::error::ColumnQError;
 use columnq::error::QueryError;
 use columnq::table::TableSource;
 use columnq::ColumnQ;
 use log::info;
+use snafu::{whatever, Whatever};
 use tokio::sync::RwLock;
 
 use crate::config::Config;
@@ -23,22 +25,25 @@ pub struct RawRoapiContext {
 }
 
 impl RawRoapiContext {
-    pub async fn new(config: &Config) -> anyhow::Result<Self> {
-        let mut cq = ColumnQ::new();
+    pub async fn new(config: &Config) -> Result<Self, Whatever> {
+        let mut cq = match config.get_datafusion_config() {
+            Ok(df_cfg) => ColumnQ::new_with_config(df_cfg),
+            _ => ColumnQ::new(),
+        };
 
         if config.tables.is_empty() && config.kvstores.is_empty() {
-            anyhow::bail!("No table nor kvstore found in config");
+            whatever!("No table nor kvstore found in config");
         }
 
         for t in config.tables.iter() {
             info!("loading `{}` as table `{}`", t.io_source, t.name);
-            cq.load_table(t).await?;
+            whatever!(cq.load_table(t).await, "Failed to load table");
             info!("registered `{}` as table `{}`", t.io_source, t.name);
         }
 
         for k in config.kvstores.iter() {
             info!("loading `{}` as kv store `{}`", k.io_source, k.name);
-            cq.load_kv(k.clone()).await?;
+            whatever!(cq.load_kv(k.clone()).await, "Failed to load kvstore");
             info!("registered `{}` as kv store `{}`", k.io_source, k.name);
         }
 
@@ -57,31 +62,36 @@ pub trait RoapiContext: Send + Sync + 'static {
 
     async fn load_table(&self, table: &TableSource) -> Result<(), ColumnQError>;
 
+    async fn schemas(&self) -> Result<Vec<(String, arrow::datatypes::SchemaRef)>, ApiErrResp>;
+
     async fn schemas_json_bytes(&self) -> Result<Vec<u8>, ApiErrResp>;
 
+    async fn table_names(&self) -> Vec<String>;
+
+    async fn table_schema(
+        &self,
+        table_name: &str,
+    ) -> Result<arrow::datatypes::SchemaRef, ApiErrResp>;
+
     async fn table_schema_json_bytes(&self, table_name: &str) -> Result<Vec<u8>, ApiErrResp>;
 
-    async fn query_graphql(
-        &self,
-        query: &str,
-    ) -> Result<Vec<arrow::record_batch::RecordBatch>, QueryError>;
+    async fn query_graphql(&self, query: &str) -> Result<Vec<RecordBatch>, QueryError>;
 
-    async fn query_sql(
-        &self,
-        query: &str,
-    ) -> Result<Vec<arrow::record_batch::RecordBatch>, QueryError>;
+    async fn query_sql(&self, query: &str) -> Result<Vec<RecordBatch>, QueryError>;
 
     async fn query_rest_table(
         &self,
         table_name: &str,
         params: &HashMap<String, String>,
-    ) -> Result<Vec<arrow::record_batch::RecordBatch>, QueryError>;
+    ) -> Result<Vec<RecordBatch>, QueryError>;
 
     async fn kv_get(&self, kv_name: &str, key: &str) -> Result<Option<String>, QueryError>;
 
-    async fn sql_to_df(&self, query: &str) -> Result<Arc<DataFrame>, DataFusionError>;
+    async fn sql_to_df(&self, query: &str) -> Result<DataFrame, DataFusionError>;
 
     async fn get_response_format(&self) -> encoding::ContentType;
+
+    async fn get_dfctx(&self) -> SessionContext;
 }
 
 #[async_trait]
@@ -98,6 +108,16 @@ impl RoapiContext for RawRoapiContext {
         ))
     }
 
+    #[inline]
+    async fn schemas(&self) -> Result<Vec<(String, arrow::datatypes::SchemaRef)>, ApiErrResp> {
+        Ok(self
+            .cq
+            .schema_map()
+            .iter()
+            .map(|(table_name, schema)| (table_name.to_string(), schema.clone()))
+            .collect())
+    }
+
     #[inline]
     async fn schemas_json_bytes(&self) -> Result<Vec<u8>, ApiErrResp> {
         serde_json::to_vec(self.cq.schema_map())
@@ -105,6 +125,24 @@ impl RoapiContext for RawRoapiContext {
             .map_err(ApiErrResp::json_serialization)
     }
 
+    #[inline]
+    async fn table_names(&self) -> Vec<String> {
+        self.cq.schema_map().keys().map(|s| s.to_string()).collect()
+    }
+
+    #[inline]
+    async fn table_schema(
+        &self,
+        table_name: &str,
+    ) -> Result<arrow::datatypes::SchemaRef, ApiErrResp> {
+        Ok(self
+            .cq
+            .schema_map()
+            .get(table_name)
+            .ok_or_else(|| ApiErrResp::not_found("invalid table name"))?
+            .clone())
+    }
+
     #[inline]
     async fn table_schema_json_bytes(&self, table_name: &str) -> Result<Vec<u8>, ApiErrResp> {
         serde_json::to_vec(
@@ -119,18 +157,12 @@ impl RoapiContext for RawRoapiContext {
     }
 
     #[inline]
-    async fn query_graphql(
-        &self,
-        query: &str,
-    ) -> Result<Vec<arrow::record_batch::RecordBatch>, QueryError> {
+    async fn query_graphql(&self, query: &str) -> Result<Vec<RecordBatch>, QueryError> {
         self.cq.query_graphql(query).await
     }
 
     #[inline]
-    async fn query_sql(
-        &self,
-        query: &str,
-    ) -> Result<Vec<arrow::record_batch::RecordBatch>, QueryError> {
+    async fn query_sql(&self, query: &str) -> Result<Vec<RecordBatch>, QueryError> {
         self.cq.query_sql(query).await
     }
 
@@ -139,7 +171,7 @@ impl RoapiContext for RawRoapiContext {
         &self,
         table_name: &str,
         params: &HashMap<String, String>,
-    ) -> Result<Vec<arrow::record_batch::RecordBatch>, QueryError> {
+    ) -> Result<Vec<RecordBatch>, QueryError> {
         self.cq.query_rest_table(table_name, params).await
     }
 
@@ -149,7 +181,7 @@ impl RoapiContext for RawRoapiContext {
     }
 
     #[inline]
-    async fn sql_to_df(&self, query: &str) -> Result<Arc<DataFrame>, DataFusionError> {
+    async fn sql_to_df(&self, query: &str) -> Result<DataFrame, DataFusionError> {
         self.cq.dfctx.sql(query).await
     }
 
@@ -157,6 +189,11 @@ impl RoapiContext for RawRoapiContext {
     async fn get_response_format(&self) -> encoding::ContentType {
         self.response_format
     }
+
+    #[inline]
+    async fn get_dfctx(&self) -> SessionContext {
+        self.cq.dfctx.clone()
+    }
 }
 
 #[async_trait]
@@ -172,6 +209,37 @@ impl RoapiContext for ConcurrentRoapiContext {
         ctx.cq.load_table(table).await
     }
 
+    #[inline]
+    async fn table_names(&self) -> Vec<String> {
+        let ctx = self.read().await;
+        ctx.cq.schema_map().keys().map(|s| s.to_string()).collect()
+    }
+
+    #[inline]
+    async fn table_schema(
+        &self,
+        table_name: &str,
+    ) -> Result<arrow::datatypes::SchemaRef, ApiErrResp> {
+        let ctx = self.read().await;
+        Ok(ctx
+            .cq
+            .schema_map()
+            .get(table_name)
+            .ok_or_else(|| ApiErrResp::not_found("invalid table name"))?
+            .clone())
+    }
+
+    #[inline]
+    async fn schemas(&self) -> Result<Vec<(String, arrow::datatypes::SchemaRef)>, ApiErrResp> {
+        let ctx = self.read().await;
+        Ok(ctx
+            .cq
+            .schema_map()
+            .iter()
+            .map(|(table_name, schema)| (table_name.to_string(), schema.clone()))
+            .collect())
+    }
+
     #[inline]
     async fn schemas_json_bytes(&self) -> Result<Vec<u8>, ApiErrResp> {
         let ctx = self.read().await;
@@ -195,19 +263,13 @@ impl RoapiContext for ConcurrentRoapiContext {
     }
 
     #[inline]
-    async fn query_graphql(
-        &self,
-        query: &str,
-    ) -> Result<Vec<arrow::record_batch::RecordBatch>, QueryError> {
+    async fn query_graphql(&self, query: &str) -> Result<Vec<RecordBatch>, QueryError> {
         let ctx = self.read().await;
         ctx.cq.query_graphql(query).await
     }
 
     #[inline]
-    async fn query_sql(
-        &self,
-        query: &str,
-    ) -> Result<Vec<arrow::record_batch::RecordBatch>, QueryError> {
+    async fn query_sql(&self, query: &str) -> Result<Vec<RecordBatch>, QueryError> {
         let ctx = self.read().await;
         ctx.cq.query_sql(query).await
     }
@@ -217,7 +279,7 @@ impl RoapiContext for ConcurrentRoapiContext {
         &self,
         table_name: &str,
         params: &HashMap<String, String>,
-    ) -> Result<Vec<arrow::record_batch::RecordBatch>, QueryError> {
+    ) -> Result<Vec<RecordBatch>, QueryError> {
         let ctx = self.read().await;
         ctx.cq.query_rest_table(table_name, params).await
     }
@@ -229,7 +291,7 @@ impl RoapiContext for ConcurrentRoapiContext {
     }
 
     #[inline]
-    async fn sql_to_df(&self, query: &str) -> Result<Arc<DataFrame>, DataFusionError> {
+    async fn sql_to_df(&self, query: &str) -> Result<DataFrame, DataFusionError> {
         let ctx = self.read().await;
         ctx.cq.dfctx.sql(query).await
     }
@@ -239,4 +301,10 @@ impl RoapiContext for ConcurrentRoapiContext {
         let ctx = self.read().await;
         ctx.response_format
     }
+
+    #[inline]
+    async fn get_dfctx(&self) -> SessionContext {
+        let ctx = self.read().await;
+        ctx.cq.dfctx.clone()
+    }
 }
